/*
 * Copyright (c) 2005-2007 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package fr.obeo.acceleo.bridge.ui.popupMenus;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

import fr.obeo.acceleo.bridge.IXmiLoader;
import fr.obeo.acceleo.bridge.XmiLoaderUtils;
import fr.obeo.acceleo.bridge.ui.AcceleoBridgeUIMessages;
import fr.obeo.acceleo.bridge.ui.AcceleoBridgeUiPlugin;
import fr.obeo.acceleo.uml14.ui.popupMenus.SpecificUml14XmiLoader;

/**
 * The operation which convert the current selected xmi from MOF to EMF.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoConvertMof2EmfOperation extends WorkspaceModifyOperation {

	/**
	 * Uml EMF file extension.
	 */
	protected String umlExtension;

	/**
	 * Selected xmi file.
	 */
	protected IFile file;

	/**
	 * Path of the new xmi. If it's null, the new file will have the same path
	 * as the MDR xmi but with the file extension "uml14".
	 */
	protected String resultPath;

	/**
	 * Constructor. The new file will have the same path as the MDR xmi but with
	 * the file extension "uml14".
	 * 
	 * @param file
	 *            is the selected xmi file
	 */
	public AcceleoConvertMof2EmfOperation(IFile file) {
		super();
		this.file = file;
		this.resultPath = null;
	}

	/**
	 * Constructor.
	 * 
	 * @param file
	 *            is the selected xmi file
	 * @param resultPath
	 *            is the path of the new xmi
	 */
	public AcceleoConvertMof2EmfOperation(IFile file, String resultPath) {
		super();
		this.file = file;
		this.resultPath = resultPath;
	}

	/* (non-Javadoc) */
	protected void execute(IProgressMonitor progressMonitor) throws CoreException {
		IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
			public void run(IProgressMonitor progressMonitor) throws CoreException {
				IXmiLoader xmiloader = XmiLoaderUtils.getXmiLoader(file);
				if (xmiloader != null) {
					try {
						xmiloader.convert(file, resultPath, progressMonitor);
					} catch (CoreException e1) {
						SpecificUml14XmiLoader specificUml14XmiLoader = new SpecificUml14XmiLoader();
						specificUml14XmiLoader.convert(file, resultPath, progressMonitor);
					}
				} else {
					AcceleoBridgeUiPlugin.getDefault().log(AcceleoBridgeUIMessages.getString("AcceleoConvertMof2EmfOperation.NoReaderFound", new Object[] { file.getFullPath().toString(), }), true); //$NON-NLS-1$
				}
			}
		};
		try {
			ResourcesPlugin.getWorkspace().run(runnable, progressMonitor);
		} catch (CoreException e) {
			AcceleoBridgeUiPlugin.getDefault().log(e, true);
		}
	}

}
